/*
This file is part of DUKIntegrator.

DUKIntegrator is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

DUKIntegrator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with DUKIntegrator.  If not, see <http://www.gnu.org/licenses/>.
 */
package ui;

import dec.LogTrace;
import dec.Options;
import dec.Params;
import general.Integrator;
import general.Main;
import java.awt.event.ItemEvent;
import pdf.*;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

public class DUKFrame extends javax.swing.JFrame implements CertificateChooser, Runnable
{
    /** Creates new form DUKFrame */
    public DUKFrame(String configPath, String xmlFile, String errFile)
    {
        initComponents();
        _configPath = configPath;
        LogTrace.init(_configPath + "emergency.log", _configPath + "emergency.trc", 0, 100, true);
        _integrator = new Integrator();
        _integrator.setChooser(this);
        _integrator.setConfigPath(_configPath);
        _xmlFile = xmlFile;
        _errFile = errFile;
        txtFileName.setText(_xmlFile);
        setVisibleValidationButton();
        populateCardList();
        getConfigDefaults();
        if(Main.createVersionFile(_integrator.getValidator(),
            _integrator.getPdfSuperCreator(), _configPath, 1) < 0)
        {
            cmbDecType.setModel(new DefaultComboBoxModel());
        }
        else
        {
            cmbDecType.setModel(new DefaultComboBoxModel(Main.getDeclarationList(_configPath)));
        }
        if(cmbDecType.getModel().getSize() != 0)
        {
            if(_defDecl != null)
            {
                cmbDecType.setSelectedItem(_defDecl);
            }
            _decType = cmbDecType.getSelectedItem().toString();
            _integrator.setDeclType(_decType);
            _zipOption = _integrator.getPdfZipOption();
            fillOptions();
            this.setTitle(_title.replaceAll("%", _decType).replaceAll("#", Main._version)
                + _integrator.getLastVersion()
                + " / " + _integrator.getPdfLastVersion());
        }
        else
        {
            JOptionPane.showMessageDialog(rootPane,
                "Nu aveti nicio declaratie instalata.");
        }
        wndChooseCertificate.setVisible(false);
        wndChooseCertificate.getRootPane().setDefaultButton(btnOK);
        wndChooseCertificate.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        wndChooseCertificate.setLocationRelativeTo(this);
        wndChooseCertificate.pack();
        pgrBar.setVisible(false);
        btnCancel.setVisible(false);
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        ctlFileChooser = new javax.swing.JFileChooser();
        wndChooseCertificate = new javax.swing.JDialog();
        jLabel1 = new javax.swing.JLabel();
        btnOK = new javax.swing.JButton();
        cmbCertificates = new javax.swing.JComboBox();
        lblFurnizor = new javax.swing.JLabel();
        lblNume = new javax.swing.JLabel();
        lblStartDate = new javax.swing.JLabel();
        lblEndDate = new javax.swing.JLabel();
        lblSerialNumber = new javax.swing.JLabel();
        txtFurnizor = new javax.swing.JTextField();
        txtNume = new javax.swing.JTextField();
        txtStartDate = new javax.swing.JTextField();
        txtEndDate = new javax.swing.JTextField();
        txtSerialNumber = new javax.swing.JTextField();
        lblHelp = new javax.swing.JLabel();
        txtFileName = new javax.swing.JTextField();
        btnGetFileName = new javax.swing.JButton();
        btnValidation = new javax.swing.JButton();
        btnPDF = new javax.swing.JButton();
        btnSign = new javax.swing.JButton();
        txtPin = new javax.swing.JPasswordField();
        lblPin = new javax.swing.JLabel();
        cmbCardType = new javax.swing.JComboBox();
        lblCardType = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        cmbDecType = new javax.swing.JComboBox();
        lblOptions = new javax.swing.JLabel();
        cmbOptions = new javax.swing.JComboBox();
        jPanel4 = new javax.swing.JPanel();
        pnlResults = new javax.swing.JScrollPane();
        txtResults = new javax.swing.JTextArea();
        pgrBar = new javax.swing.JProgressBar();
        btnCancel = new javax.swing.JButton();

        wndChooseCertificate.setTitle("Alegeti un certficat pt. semnare");
        wndChooseCertificate.setModal(true);
        wndChooseCertificate.setResizable(false);

        jLabel1.setText("Token cu mai multe certificate. Selectati certificatul dorit");

        btnOK.setText("OK");
        btnOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOKActionPerformed(evt);
            }
        });

        cmbCertificates.setAutoscrolls(true);
        cmbCertificates.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbCertificatesActionPerformed(evt);
            }
        });

        lblFurnizor.setText("Furnizor:");

        lblNume.setText("Nume:");

        lblStartDate.setText("StartValabilitate:");

        lblEndDate.setText("EndValabilitate:");

        lblSerialNumber.setText("SerialNumber:");

        txtFurnizor.setEditable(false);

        txtNume.setEditable(false);

        txtStartDate.setEditable(false);

        txtEndDate.setEditable(false);

        txtSerialNumber.setEditable(false);

        javax.swing.GroupLayout wndChooseCertificateLayout = new javax.swing.GroupLayout(wndChooseCertificate.getContentPane());
        wndChooseCertificate.getContentPane().setLayout(wndChooseCertificateLayout);
        wndChooseCertificateLayout.setHorizontalGroup(
            wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(wndChooseCertificateLayout.createSequentialGroup()
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(wndChooseCertificateLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(cmbCertificates, 0, 441, Short.MAX_VALUE))
                    .addGroup(wndChooseCertificateLayout.createSequentialGroup()
                        .addGap(56, 56, 56)
                        .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 272, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
            .addGroup(wndChooseCertificateLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(lblSerialNumber, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblEndDate, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblStartDate, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblNume, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(lblFurnizor, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 81, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(txtFurnizor, javax.swing.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE)
                    .addComponent(txtNume, javax.swing.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE)
                    .addComponent(txtStartDate, javax.swing.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE)
                    .addComponent(txtEndDate, javax.swing.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE)
                    .addComponent(txtSerialNumber, javax.swing.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE))
                .addContainerGap())
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, wndChooseCertificateLayout.createSequentialGroup()
                .addContainerGap(216, Short.MAX_VALUE)
                .addComponent(btnOK)
                .addGap(196, 196, 196))
        );
        wndChooseCertificateLayout.setVerticalGroup(
            wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(wndChooseCertificateLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addGap(28, 28, 28)
                .addComponent(cmbCertificates, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblFurnizor)
                    .addComponent(txtFurnizor, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(11, 11, 11)
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblNume)
                    .addComponent(txtNume, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(11, 11, 11)
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblStartDate)
                    .addComponent(txtStartDate, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(11, 11, 11)
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblEndDate)
                    .addComponent(txtEndDate, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(wndChooseCertificateLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblSerialNumber)
                    .addComponent(txtSerialNumber, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 17, Short.MAX_VALUE)
                .addComponent(btnOK)
                .addContainerGap())
        );

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Procesare XML");
        setName("frmMain"); // NOI18N
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        lblHelp.setText("introduceti nume fisier XML sau nume folder (pt. a prelucra toate fisierele XML din el) sau folositi butonul <Alege fisiere>");

        txtFileName.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                txtFileNameKeyReleased(evt);
            }
        });

        btnGetFileName.setText("Alege fisiere");
        btnGetFileName.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                btnGetFileNameMouseClicked(evt);
            }
        });

        btnValidation.setText("Validare");
        btnValidation.setEnabled(false);
        btnValidation.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                btnValidationMouseClicked(evt);
            }
        });

        btnPDF.setText("Validare + creare PDF");
        btnPDF.setEnabled(false);
        btnPDF.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                btnPDFMouseClicked(evt);
            }
        });

        btnSign.setText("Validare + creare PDF semnat");
        btnSign.setEnabled(false);
        btnSign.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                btnSignMouseClicked(evt);
            }
        });

        txtPin.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                txtPinKeyReleased(evt);
            }
        });

        lblPin.setText("introduceti pinul pentru semnare:");

        cmbCardType.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbCardTypeActionPerformed(evt);
            }
        });

        lblCardType.setText("selectati tipul de Smart Card");

        jLabel2.setText("alegeti tipul declaratiei");

        cmbDecType.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cmbDecType.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbDecTypeActionPerformed(evt);
            }
        });

        lblOptions.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        lblOptions.setText("alegeti optiuni validare");

        cmbOptions.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cmbOptionsItemStateChanged(evt);
            }
        });

        txtResults.setColumns(20);
        txtResults.setEditable(false);
        txtResults.setFont(new java.awt.Font("Times New Roman", 0, 12));
        txtResults.setLineWrap(true);
        txtResults.setRows(5);
        pnlResults.setViewportView(txtResults);

        btnCancel.setText("Anulare");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addComponent(pgrBar, javax.swing.GroupLayout.PREFERRED_SIZE, 460, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(btnCancel, javax.swing.GroupLayout.DEFAULT_SIZE, 113, Short.MAX_VALUE))
            .addComponent(pnlResults, javax.swing.GroupLayout.DEFAULT_SIZE, 591, Short.MAX_VALUE)
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel4Layout.createSequentialGroup()
                .addComponent(pnlResults, javax.swing.GroupLayout.DEFAULT_SIZE, 411, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(pgrBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(btnCancel, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(lblHelp, javax.swing.GroupLayout.DEFAULT_SIZE, 582, Short.MAX_VALUE)
                        .addContainerGap(19, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(txtFileName, javax.swing.GroupLayout.DEFAULT_SIZE, 492, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnGetFileName)
                        .addContainerGap())
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(cmbCardType, javax.swing.GroupLayout.PREFERRED_SIZE, 186, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(lblCardType, javax.swing.GroupLayout.PREFERRED_SIZE, 182, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                    .addComponent(txtPin)
                                    .addComponent(lblPin, javax.swing.GroupLayout.PREFERRED_SIZE, 163, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(btnSign, javax.swing.GroupLayout.PREFERRED_SIZE, 222, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                .addComponent(cmbDecType, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(cmbOptions, 0, 237, Short.MAX_VALUE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(btnValidation)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(btnPDF, javax.swing.GroupLayout.PREFERRED_SIZE, 139, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addContainerGap())
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 115, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(lblOptions, javax.swing.GroupLayout.PREFERRED_SIZE, 216, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(lblHelp)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(txtFileName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnGetFileName))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(lblOptions))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnPDF)
                    .addComponent(btnValidation)
                    .addComponent(cmbDecType, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(cmbOptions, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lblPin)
                    .addComponent(lblCardType))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnSign)
                    .addComponent(txtPin, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(cmbCardType, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    public Sign.CertAlias chooseCertificate(List coll)
    {
        DefaultComboBoxModel cmbModel = new DefaultComboBoxModel();
        for(Object cert : coll)
        {
            cmbModel.addElement(cert);
        }
        cmbCertificates.setModel(cmbModel);
        changeStatusCombo();
        wndChooseCertificate.setVisible(true);
        return (Sign.CertAlias)cmbCertificates.getSelectedItem();
    }

    private void populateCardList()
    {
        if(_configPath.equals("") == true)
        {
            return;
        }
        File cfg = new File(_configPath);
        if(cfg.isDirectory() == false)
        {
            return;
        }
        String[] cfgList = cfg.list(new FilenameFilter()
        {
            public boolean accept(File dir, String name)
            {
                return name.endsWith(".cfg");
            }
        });
        if(cfgList == null || cfgList.length == 0)
        {
            return;
        }
        cmbCardType.addItem(_AUTO_DETECT);
        for(int i = 0; i < cfgList.length; i++)
        {
            cmbCardType.addItem(cfgList[i].substring(0, cfgList[i].length() - 4));
        }
//        cmbCardType.setSelectedIndex(-1);
        cmbCardType.setSelectedIndex(0);
    }

    private int validateXML(String fileName)
    {
        int returns;
        try
        {
            returns = _integrator.parseDocument(fileName, null);
            txtResults.append(_integrator.getFinalMessage());
            return returns;
        }
        catch(Throwable e)
        {
            txtResults.append("Eroare de deployment!" + _newLine);
            txtResults.append("       mesaj: " + e.toString() + _newLine);
            return -2;
        }
    }

    private int pdfCreation(String fileName)
    {
        int ret = 0;
        try
        {
            ret = _integrator.pdfCreation(fileName, null, null, null);
            txtResults.append(_integrator.getFinalMessage());
            return ret;
        }
        catch(Throwable e)
        {
            txtResults.append("Eroare de deployment!" + _newLine);
            txtResults.append("       mesaj: " + e.toString() + _newLine);
            return -2;
        }
    }

    private int pdfSigning(String fileName)
    {
        int ret = 0;
        try
        {
            String msg;
            String smartCard = (String)cmbCardType.getSelectedItem();
            if(smartCard.equals(_AUTO_DETECT) == false)
            {
                ret = _integrator.signPdf(fileName, null, null,
                    null, txtPin.getText(), smartCard);
                msg = _integrator.getFinalMessage();
                txtResults.append(msg);
                return ret;
            }
            else
            {
                int i = 0;
                for(; i < cmbCardType.getModel().getSize(); i++)
                {
                    smartCard = (String)cmbCardType.getModel().getElementAt(i);
                    if(smartCard.equals(_AUTO_DETECT) == true)
                    {
                        continue;
                    }
                    ret = _integrator.signPdf(fileName, null, null,
                        null, txtPin.getText(), smartCard);
                    msg = _integrator.getFinalMessage();
                    if(ret < 0)
                    {
                        continue;
                    }
                    else
                    {
                        txtResults.append(msg);
                        cmbCardType.setSelectedIndex(i);
                        return 0;
                    }
                }
                txtResults.append("Nu am putut detecta tipul SmartCardului!" + _newLine);
                txtResults.append("       " + "Fie este un tip nou, necunoscut, fie pinul introdus nu este corect" + _newLine);
                return ret;
            }
        }
        catch(Throwable e)
        {
            txtResults.append("Eroare de deployment!" + _newLine);
            txtResults.append("       mesaj: " + e.toString() + _newLine);
            return -2;
        }
    }

    private void enableUI(boolean flag)
    {
        cmbCardType.setEnabled(flag);
        cmbDecType.setEnabled(flag);
        btnValidation.setEnabled(flag);
        btnPDF.setEnabled(flag);
        txtPin.setEnabled(flag);
        txtFileName.setEnabled(flag);
        btnGetFileName.setEnabled(flag);
        if(flag == false)
        {
            _getEnableOptions = cmbOptions.isEnabled();
            _getEnableSign = btnSign.isEnabled();
            cmbOptions.setEnabled(flag);
            btnSign.setEnabled(flag);
        }
        else
        {
            cmbOptions.setEnabled(_getEnableOptions);
            btnSign.setEnabled(_getEnableSign);
        }
    }

    private void enableUIProgBar(boolean flag, int max)
    {
        pgrBar.setVisible(!flag);
        btnCancel.setVisible(!flag);
        _stopProcessing = flag;
        if(flag == false)
        {
            pgrBar.setMinimum(0);
            pgrBar.setMaximum(max);
            pgrBar.setValue(0);
        }
    }

    private void processFiles(int type)
    {
        _processingType = type;
        enableUI(false);
        Thread th = new Thread(this);
        th.start();
    }

    public void run()
    {
        int ret = 0;
        File file = new File(_xmlFile);
        File[] fileList = null;
        if(file.isDirectory() == true)
        {
            try
            {
                fileList = file.listFiles(_crtFilter);
            }
            catch(Throwable ex)
            {
                fileList = null;
            }
//            fileList = file.listFiles(new FilenameFilter()
//            {
//
//                public boolean accept(File dir, String name)
//                {
//                    return name.toLowerCase().endsWith(".xml");
//                }
//            });
            enableUIProgBar(false, fileList.length);
            for(int i = 0; i < fileList.length; i++)
            {
                if(_stopProcessing == true)
                {
                    break;
                }
                switch(_processingType)
                {
                    case 0:
                        ret = validateXML(fileList[i].getPath());
                        break;
                    case 1:
                        ret = pdfCreation(fileList[i].getPath());
                        break;
                    case 2:
                        ret = pdfSigning(fileList[i].getPath());
                        break;
                }
                if(ret < -100)
                {
                    break;
                }
                pgrBar.setValue(i + 1);
            }
            enableUIProgBar(true, 0);
        }
        else if(file.isFile() == true)
        {
            switch(_processingType)
            {
                case 0:
                    ret = validateXML(_xmlFile);
                    break;
                case 1:
                    ret = pdfCreation(_xmlFile);
                    break;
                case 2:
                    ret = pdfSigning(_xmlFile);
                    break;
            }
        }
        else
        {
            JOptionPane.showMessageDialog(rootPane, "introduceti corect calea si numele fisierului");
        }
        enableUI(true);
    }

    private void signStatus()
    {
        if(txtFileName.getText().equals("") == false
            && txtPin.getText().equals("") == false
            && cmbCardType.getSelectedIndex() >= 0)
        {
            btnSign.setEnabled(true);
        }
        else
        {
            btnSign.setEnabled(false);
        }
    }

    private void changeStatusCombo()
    {
        Sign.CertAlias certAlias = (Sign.CertAlias)cmbCertificates.getSelectedItem();
        X509Certificate cert = certAlias._cert;
        txtFurnizor.setText(cert.getIssuerDN().getName());
        txtFurnizor.setCaretPosition(0);
        txtNume.setText(cert.getSubjectDN().getName());
        txtNume.setCaretPosition(0);
        txtStartDate.setText(dateFormat.format(cert.getNotBefore()));
        txtEndDate.setText(dateFormat.format(cert.getNotAfter()));
        txtSerialNumber.setText(cert.getSerialNumber().toString());
        txtSerialNumber.setCaretPosition(0);
    }

    private void fillOptions()
    {
        cmbOptions.removeAllItems();
        Options[] options = _integrator.getOptions();
        if(options == null)
        {
            cmbOptions.setEnabled(false);
        }
        else
        {
            cmbOptions.setModel(new DefaultComboBoxModel(options));
            try
            {
                String opt = Params.getProperty("_" + _decType);
                for(int i = 0; i < options.length; i++)
                {
                    if(options[i]._name.equals(opt) == true)
                    {
                        _flag = false;
                        cmbOptions.setSelectedIndex(i);
                        _flag = true;
                        break;
                    }
                }
            }
            catch(Throwable ex)
            {
            }
            cmbOptions.setEnabled(true);
        }
    }

    private void setVisibleValidationButton()
    {
        if(txtFileName.getText().equals("") == false)
        {
            btnValidation.setEnabled(true);
            btnPDF.setEnabled(true);
        }
        else
        {
            btnValidation.setEnabled(false);
            btnPDF.setEnabled(false);
        }
        signStatus();
    }

    private void getConfigDefaults()
    {
        if(_configPath.equals("") == true)
        {
            return;
        }
        try
        {
//            _props = new Properties();
//            FileInputStream file = new FileInputStream(_configPath + "config.properties");
//            _props.load(file);
//            file.close();
            Params.init(_configPath + "config.properties");
            String folder = Params.getProperty("defFolder");
            if(folder != null)
            {
                ctlFileChooser.setCurrentDirectory(new File(folder));
            }
            String card = Params.getProperty("defSmartCard");
            if(card != null)
            {
                cmbCardType.setSelectedItem(card);
            }
            _urlVersiuni = Params.getProperty("urlVersiuni");
            _proxy = Params.getProperty("proxy");
            _proxyIP = Params.getProperty("proxyIP");
            _proxyPort = Params.getProperty("proxyPort");
            _defDecl = Params.getProperty("tipDeclaratie");
        }
        //catch exception in case properties file does not exist
        catch(Throwable e)
        {
            return;
        }
    }

    private void setConfigDefaults()
    {
        FileOutputStream file = null;
        try
        {
//            if(_props == null)
//            {
//                _props = new Properties();
//            }
            Params.setProperty("defFolder", ctlFileChooser.getCurrentDirectory().getCanonicalPath());
            if(cmbCardType.getSelectedIndex() >= 0)
            {
                Params.setProperty("defSmartCard", cmbCardType.getSelectedItem().toString());
            }
            Params.setProperty("tipDeclaratie", _decType);
            Params.close();
//            file = new FileOutputStream(_configPath + "config.properties");
//            _props.store(file, "pentru a comenta o linie, cu pastrarea ei dupa salvare, inserati un 'x' la inceputul liniei");
        }
        //catch exception in case properties file does not exist
        catch(Throwable e)
        {
        }
        finally
        {
            try
            {
                file.close();
            }
            catch(Throwable e)
            {
            }
        }
    }

    public static String adjustString(String par)
    {
        return (par == null) ? "" : par;
    }

    public String chooseZipFile(String xmlFile, int zipOption)
    {
        return getZipFile(xmlFile, zipOption);
    }

    public String getZipFile(String xmlFile, int zipOption)
    {
        String zipFile = null;
        String text = null;
        int rez = 0;
        if(zipOption < 0)
        {
            return "";
        }
        else if(zipOption == 0)
        {
            text = "(optional)";
        }
        else
        {
            text = "(obligatoriu)";
        }
        if(xmlFile.toLowerCase().endsWith(".xml") == true)
        {
            zipFile = xmlFile.substring(0, xmlFile.length() - 4) + ".zip";
        }
        else
        {
            zipFile = xmlFile + ".zip";
        }
        File file = new File(zipFile);
        if(file.exists() == true)
        {
//            ctlFileChooser.setSelectedFile(file);
        }
        ctlFileChooser.setDialogTitle("alegeti un fisier zip pt. atasare la fisierul pdf "
            + text);
        ctlFileChooser.resetChoosableFileFilters();
        ctlFileChooser.setFileFilter(new FileNameExtensionFilter("fisiere ZIP", "zip"));
        ctlFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        ctlFileChooser.setAcceptAllFileFilterUsed(false);
        if((rez = ctlFileChooser.showOpenDialog(this)) == JFileChooser.APPROVE_OPTION)
        {
            return ctlFileChooser.getSelectedFile().getPath();
        }
        if(zipOption == 0)
        {
            return "";
        }
        else
        {
            //eroare: zip obligatoriu dar selectia refuzata
            return null;
        }
    }

    private void btnValidationMouseClicked(java.awt.event.MouseEvent evt)//GEN-FIRST:event_btnValidationMouseClicked
    {//GEN-HEADEREND:event_btnValidationMouseClicked
        if(btnValidation.isEnabled() == true)
        {
            processFiles(0);
        }
    }//GEN-LAST:event_btnValidationMouseClicked

    private void btnGetFileNameMouseClicked(java.awt.event.MouseEvent evt)//GEN-FIRST:event_btnGetFileNameMouseClicked
    {//GEN-HEADEREND:event_btnGetFileNameMouseClicked
        if(btnGetFileName.isEnabled() == false)
        {
            return;
        }
        ctlFileChooser.setDialogTitle("Alegeti un XML sau un PDF sau un FOLDER (pentru toate fisierele XML si PDF)");
        ctlFileChooser.resetChoosableFileFilters();
//        ctlFileChooser.setFileFilter(new FileNameExtensionFilter("fisiere XML", "xml"));
        ctlFileChooser.resetChoosableFileFilters();
        ctlFileChooser.setAcceptAllFileFilterUsed(false);
        FileFilter filter = null;
        ctlFileChooser.addChoosableFileFilter(filter = new ExtensionFileFilter(
            new String[]
            {
                ".XML"
            },
            "Fisiere XML (*.XML)"));
        ctlFileChooser.addChoosableFileFilter(new ExtensionFileFilter(
            new String[]
            {
                ".PDF"
            },
            "Fisiere PDF (*.PDF)"));
        ctlFileChooser.addChoosableFileFilter(new ExtensionFileFilter(
            new String[]
            {
                ".XML", ".PDF"
            },
            "Fisiere XML si PDF (*.XML|PDF)"));
        ctlFileChooser.setFileFilter(filter);
        ctlFileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
//        ctlFileChooser.setAcceptAllFileFilterUsed(false);
        ctlFileChooser.setSelectedFile(null);
        if(ctlFileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
        {
            _xmlFile = ctlFileChooser.getSelectedFile().getPath();
            _crtFilter = (java.io.FileFilter)ctlFileChooser.getFileFilter();
            txtFileName.setText(_xmlFile);
            setVisibleValidationButton();
        }
    }//GEN-LAST:event_btnGetFileNameMouseClicked

    private void txtFileNameKeyReleased(java.awt.event.KeyEvent evt)//GEN-FIRST:event_txtFileNameKeyReleased
    {//GEN-HEADEREND:event_txtFileNameKeyReleased
        setVisibleValidationButton();
        _xmlFile = txtFileName.getText();
    }//GEN-LAST:event_txtFileNameKeyReleased

    private void btnPDFMouseClicked(java.awt.event.MouseEvent evt)//GEN-FIRST:event_btnPDFMouseClicked
    {//GEN-HEADEREND:event_btnPDFMouseClicked
        if(btnPDF.isEnabled() == true)
        {
            processFiles(1);
        }
    }//GEN-LAST:event_btnPDFMouseClicked

    private void btnSignMouseClicked(java.awt.event.MouseEvent evt)//GEN-FIRST:event_btnSignMouseClicked
    {//GEN-HEADEREND:event_btnSignMouseClicked
        if(btnSign.isEnabled() == true)
        {
            processFiles(2);
        }
    }//GEN-LAST:event_btnSignMouseClicked

    private void txtPinKeyReleased(java.awt.event.KeyEvent evt)//GEN-FIRST:event_txtPinKeyReleased
    {//GEN-HEADEREND:event_txtPinKeyReleased
        signStatus();
        _integrator.setNoCertificate();
    }//GEN-LAST:event_txtPinKeyReleased

    private void cmbCardTypeActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cmbCardTypeActionPerformed
    {//GEN-HEADEREND:event_cmbCardTypeActionPerformed
        signStatus();
        _integrator.setNoCertificate();
    }//GEN-LAST:event_cmbCardTypeActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowClosing
    {//GEN-HEADEREND:event_formWindowClosing
        setConfigDefaults();
    }//GEN-LAST:event_formWindowClosing

    private void btnOKActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnOKActionPerformed
    {//GEN-HEADEREND:event_btnOKActionPerformed
        //System.out.println("OK " + (String)cmbCertificates.getSelectedItem());
        wndChooseCertificate.setVisible(false);
    }//GEN-LAST:event_btnOKActionPerformed

    private void cmbCertificatesActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cmbCertificatesActionPerformed
    {//GEN-HEADEREND:event_cmbCertificatesActionPerformed
        // TODO add your handling code here:
        changeStatusCombo();
    }//GEN-LAST:event_cmbCertificatesActionPerformed

    private void cmbDecTypeActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cmbDecTypeActionPerformed
    {//GEN-HEADEREND:event_cmbDecTypeActionPerformed
        // TODO add your handling code here:
        _decType = cmbDecType.getSelectedItem().toString();
        _integrator.setDeclType(_decType);
        _zipOption = _integrator.getPdfZipOption();
        this.setTitle(_title.replaceAll("%", _decType).replaceAll("#", Main._version)
            + _integrator.getLastVersion()
            + " / " + _integrator.getPdfLastVersion());
        fillOptions();
    }//GEN-LAST:event_cmbDecTypeActionPerformed

    private void cmbOptionsItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_cmbOptionsItemStateChanged
    {//GEN-HEADEREND:event_cmbOptionsItemStateChanged
        // TODO add your handling code here:
        if(evt.getStateChange() == ItemEvent.SELECTED)
        {
            try
            {
                Options option = (Options)cmbOptions.getSelectedItem();
                String opt = adjustString(option._comments);
                if(opt.equals("") == false && _flag == true)
                {
                    cmbOptions.hidePopup();
                    cmbDecType.hidePopup();
                    JOptionPane.showMessageDialog(rootPane, opt, "Atentie!", JOptionPane.WARNING_MESSAGE);
//                    System.out.println("salut");
                }
                Params.setProperty("_" + _decType, option._name);
                _integrator.useOptions(option._options);
            }
            catch(Throwable e)
            {
            }
        }
    }//GEN-LAST:event_cmbOptionsItemStateChanged

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnCancelActionPerformed
    {//GEN-HEADEREND:event_btnCancelActionPerformed
        // TODO add your handling code here:
        _stopProcessing = true;
    }//GEN-LAST:event_btnCancelActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnGetFileName;
    private javax.swing.JButton btnOK;
    private javax.swing.JButton btnPDF;
    private javax.swing.JButton btnSign;
    private javax.swing.JButton btnValidation;
    private javax.swing.JComboBox cmbCardType;
    private javax.swing.JComboBox cmbCertificates;
    private javax.swing.JComboBox cmbDecType;
    private javax.swing.JComboBox cmbOptions;
    private javax.swing.JFileChooser ctlFileChooser;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JLabel lblCardType;
    private javax.swing.JLabel lblEndDate;
    private javax.swing.JLabel lblFurnizor;
    private javax.swing.JLabel lblHelp;
    private javax.swing.JLabel lblNume;
    private javax.swing.JLabel lblOptions;
    private javax.swing.JLabel lblPin;
    private javax.swing.JLabel lblSerialNumber;
    private javax.swing.JLabel lblStartDate;
    private javax.swing.JProgressBar pgrBar;
    private javax.swing.JScrollPane pnlResults;
    private javax.swing.JTextField txtEndDate;
    private javax.swing.JTextField txtFileName;
    private javax.swing.JTextField txtFurnizor;
    private javax.swing.JTextField txtNume;
    private javax.swing.JPasswordField txtPin;
    private javax.swing.JTextArea txtResults;
    private javax.swing.JTextField txtSerialNumber;
    private javax.swing.JTextField txtStartDate;
    private javax.swing.JDialog wndChooseCertificate;
    // End of variables declaration//GEN-END:variables
    static public String _newLine = System.getProperty("line.separator");
    static private final String _AUTO_DETECT = "*autoDetect";
    static private final String _title = "DUKIntegrator - versiune: # - % - versiune validator: ";
    private SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MMM-yyyy");
    private String _configPath = "";
    private String _xmlFile = "";
    private String _errFile = "";
//    private Properties _props = null;
    private boolean _flag = true;
    private String _decType = null;
    private String _defDecl = null;
    private Integrator _integrator = null;
    public String _urlVersiuni = null;
    public String _proxy = null;
    public String _proxyIP = null;
    public String _proxyPort = null;
    private int _processingType;
    private boolean _stopProcessing = false;
    private boolean _getEnableOptions;
    private boolean _getEnableSign;
    private int _zipOption;
    private java.io.FileFilter _crtFilter = null;

    //clasa extinde o clasa abstracta si implementeaza o interfata care au
    //  in comun o metoda (accept()) cu aceeasi semnatura. Aceasta metoda
    //  admite, din fericire, aceeasi implementare (altfel convietuirea celor
    //  2 clasa/interfata nu pare posibila). Metoda este definita o singura data
    static class ExtensionFileFilter extends FileFilter
        implements java.io.FileFilter
    {
        private java.util.List<String> extensions;
        private String description;

        public ExtensionFileFilter(String[] exts, String desc)
        {
            if(exts != null)
            {
                extensions = new java.util.ArrayList<String>();

                for(String ext : exts)
                {
                    extensions.add(ext.replace(".", "").trim().toLowerCase());
                }
            }
            description = (desc != null) ? desc.trim() : "";
        }

        @Override
        public boolean accept(File f)
        {

            if(f.isDirectory())
            {
                return true;
            }
            if(extensions == null)
            {
                return false;
            }
            for(String ext : extensions)
            {
                if(f.getName().toLowerCase().endsWith("." + ext))
                {
                    return true;
                }
            }
            return false;
        }

        @Override
        public String getDescription()
        {
            return description;
        }
    }
}
